/**
 * @file Run apeman-task-contrib-chmod
 */

"use strict";

var argx = require('argx'),
    async = require('async'),
    fs = require('fs'),
    glob = require('glob'),
    objnest = require('objnest');

/** @lends apemanTaskContribChmod */
function apemanTaskContribChmod(expression, options) {
    var args = argx(arguments);
    expression = [].concat(args.shift('string|array') || []);
    options = objnest.expand(args.pop('object') || {});

    return function task(context, callback) {
        var logger = context.logger;

        async.eachSeries(expression, function (expression, callback) {
            var values = expression.split(/[\s,]/g);
            var mode = values.shift(), pattern = values.pop();
            var originMode, changedMode;
            _eachFiles(pattern, function (filename, callback) {
                async.series([
                    function (callback) {
                        fs.stat(filename, function (err, stat) {
                            originMode = !err && stat.mode;
                            callback(err);
                        });
                    },
                    function (callback) {
                        fs.chmod(filename, mode, callback);
                    },
                    function (callback) {
                        fs.stat(filename, function (err, stat) {
                            changedMode = !err && stat.mode;
                            callback(err);
                        });
                    },
                    function (callback) {
                        var changed = originMode !== changedMode;
                        if (changed) {
                            logger.debug('Permission changed: %s, %s', filename, mode);
                        }
                        callback(null);
                    }
                ], callback);
            }, callback);
        }, callback);
    }
}

function _eachFiles(pattern, handler, callback) {
    async.waterfall([
        function (callback) {
            glob(pattern, callback);
        },
        function (filenames, callback) {
            async.eachSeries(filenames, handler, callback);
        }
    ], callback);
}


module.exports = apemanTaskContribChmod;
